/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"
%{C++
namespace mozilla {
class TimeStamp;
}
%}

native TimeStamp(mozilla::TimeStamp);

// All properties return zero if the value is not available
[scriptable, uuid(E94AB245-B40D-4154-8B7F-B6E0F2461031)]
interface nsITimedChannel : nsISupports {
  // Set this attribute to true to enable collection of timing data.
  // channelCreationTime will be available even with this attribute set to
  // false.
  attribute boolean timingEnabled;

  // The number of redirects
  attribute uint16_t redirectCount;

  [noscript] readonly attribute TimeStamp channelCreation;
  [noscript] readonly attribute TimeStamp asyncOpen;

  // The following are only set when the document is not (only) read from the
  // cache
  [noscript] readonly attribute TimeStamp domainLookupStart;
  [noscript] readonly attribute TimeStamp domainLookupEnd;
  [noscript] readonly attribute TimeStamp connectStart;
  [noscript] readonly attribute TimeStamp connectEnd;
  [noscript] readonly attribute TimeStamp requestStart;
  [noscript] readonly attribute TimeStamp responseStart;
  [noscript] readonly attribute TimeStamp responseEnd;

  // The redirect attributes timings must be writeble, se we can transfer
  // the data from one channel to the redirected channel.
  [noscript] attribute TimeStamp redirectStart;
  [noscript] attribute TimeStamp redirectEnd;

  // The initiator type
  [noscript] attribute AString initiatorType;

  // This flag should be set to false only if a cross-domain redirect occurred
  [noscript] attribute boolean allRedirectsSameOrigin;

  // The following are only set if the document is (partially) read from the
  // cache
  [noscript] readonly attribute TimeStamp cacheReadStart;
  [noscript] readonly attribute TimeStamp cacheReadEnd;

  // All following are PRTime versions of the above.
  readonly attribute PRTime channelCreationTime;
  readonly attribute PRTime asyncOpenTime;
  readonly attribute PRTime domainLookupStartTime;
  readonly attribute PRTime domainLookupEndTime;
  readonly attribute PRTime connectStartTime;
  readonly attribute PRTime connectEndTime;
  readonly attribute PRTime requestStartTime;
  readonly attribute PRTime responseStartTime;
  readonly attribute PRTime responseEndTime;
  readonly attribute PRTime cacheReadStartTime;
  readonly attribute PRTime cacheReadEndTime;
  readonly attribute PRTime redirectStartTime;
  readonly attribute PRTime redirectEndTime;
};
